@@ -22,11 +22,46 @@ use crate::{Error, Result};
 pub fn record_artifact(
     path: &str,
     hash_algorithms: &[HashAlgorithm],
+    lstrip_paths: Option<&[&str]>,
 ) -> Result<(VirtualTargetPath, TargetDescription)> {
     let file = File::open(path)?;
     let mut reader = BufReader::new(file);
     let (_length, hashes) = crypto::calculate_hashes(&mut reader, hash_algorithms)?;
-    Ok((VirtualTargetPath::new(String::from(path))?, hashes))
+    let lstripped_path = apply_left_strip(path, lstrip_paths)?;
+    Ok((VirtualTargetPath::new(lstripped_path)?, hashes))
+}
+
+/// Given an artifact path in `&str` format, left strip path for given artifact based an optional array of `lstrip_paths` provided,
+/// returning the stripped file path in String format wrapped in `Result`.
+fn apply_left_strip(path: &str, lstrip_paths: Option<&[&str]>) -> Result<String> {
+    // If lstrip_paths is None, skip strip.
+    // Else, check if path starts with any given lstrip paths and strip
+    if lstrip_paths.is_none() {
+        return Ok(String::from(path));
+    }
+    let l_paths = lstrip_paths.unwrap();
+    let mut stripped_path = path;
+    let mut find_prefix = "";
+    for l_path in l_paths.iter() {
+        if !path.starts_with(l_path) {
+            continue;
+        }
+        // if find possible prefix longer than
+        if !find_prefix.is_empty() && find_prefix.len() >= l_path.len() {
+            continue;
+        }
+        stripped_path = path.strip_prefix(l_path).ok_or_else(|| {
+            Error::from(io::Error::new(
+                std::io::ErrorKind::Other,
+                format!(
+                    "Lstrip Error: error stripping {} from path {}",
+                    l_path, path
+                ),
+            ))
+        })?;
+        find_prefix = l_path;
+    }
+    Ok(String::from(stripped_path))
 }
 
 /// Traverses through the passed array of paths, hashes the content of files
@@ -36,6 +71,7 @@ pub fn record_artifact(
 ///
 /// * `paths` - An array of string slices (`&str`) that holds the paths to be traversed. If a symbolic link cycle is detected in the `paths` during traversal, it is skipped.
 /// * `hash_algorithms` - An array of string slice (`&str`) wrapped in an `Option` that holds the hash algorithms to be used. If `None` is provided, Sha256 is assumed as default.
+/// * `lstrip_paths` - An array of string slice (`&str`) wrapped in an `Option` that is left stripped from the path of every artifact that contains it.
 ///
 /// # Examples
 ///
@@ -43,11 +79,12 @@ pub fn record_artifact(
 /// // You can have rust code between fences inside the comments
 /// // If you pass --test to `rustdoc`, it will even test it for you!
 /// # use in_toto::runlib::{record_artifacts};
-/// let materials = record_artifacts(&["tests/test_runlib"], None).unwrap();
+/// let materials = record_artifacts(&["tests/test_runlib"], None, None).unwrap();
 /// ```
 pub fn record_artifacts(
     paths: &[&str],
     hash_algorithms: Option<&[&str]>,
+    lstrip_paths: Option<&[&str]>,
 ) -> Result<BTreeMap<VirtualTargetPath, TargetDescription>> {
     // Verify hash_algorithms inputs are valid
     let available_algorithms = HashAlgorithm::return_all();
@@ -91,14 +128,27 @@ pub fn record_artifacts(
                     };
                     if symlink_metadata(&s_path)?.file_type().is_file() {
                         let (virtual_target_path, hashes) =
-                            record_artifact(&path, hash_algorithms)?;
+                            record_artifact(&path, hash_algorithms, lstrip_paths)?;
+                        if artifacts.contains_key(&virtual_target_path) {
+                            return Err(Error::LinkGatheringError(format!(
+                                "non unique stripped path {}",
+                                virtual_target_path.to_string()
+                            )));
+                        }
                         artifacts.insert(virtual_target_path, hashes);
                     }
                 }
             }
             // If entry is a file, open and hash the file
             if file_type.is_file() {
-                let (virtual_target_path, hashes) = record_artifact(&path, hash_algorithms)?;
+                let (virtual_target_path, hashes) =
+                    record_artifact(&path, hash_algorithms, lstrip_paths)?;
+                if artifacts.contains_key(&virtual_target_path) {
+                    return Err(Error::LinkGatheringError(format!(
+                        "non unique stripped path {}",
+                        virtual_target_path.to_string()
+                    )));
+                }
                 artifacts.insert(virtual_target_path, hashes);
             }
         }
@@ -216,6 +266,7 @@ pub fn run_command(cmd_args: &[&str], run_dir: Option<&str>) -> Result<BTreeMap<
 /// * `cmd_args` - A string slice (`&str`) where the first element is a command and the remaining elements are arguments passed to that command.
 /// * `key` -  A key used to sign the resulting link metadata.
 /// * `hash_algorithms` - An array of string slice (`&str`) wrapped in an `Option` that holds the hash algorithms to be used. If `None` is provided, Sha256 is assumed as default.
+/// * `lstrip_paths` - An array of string slice (`&str`) wrapped in an `Option` that is left stripped from the path of every artifact that contains it.
 ///
 /// # Examples
 ///
@@ -226,7 +277,7 @@ pub fn run_command(cmd_args: &[&str], run_dir: Option<&str>) -> Result<BTreeMap<
 /// # use in_toto::crypto::PrivateKey;
 /// const ED25519_1_PRIVATE_KEY: &'static [u8] = include_bytes!("../tests/ed25519/ed25519-1");
 /// let key = PrivateKey::from_ed25519(ED25519_1_PRIVATE_KEY).unwrap();
-/// let link = in_toto_run("example", Some("tests"), &["tests/test_runlib"], &["tests/test_runlib"],  &["sh", "-c", "echo 'in_toto says hi' >> hello_intoto"], Some(&key), Some(&["sha512", "sha256"]),).unwrap();
+/// let link = in_toto_run("example", Some("tests"), &["tests/test_runlib"], &["tests/test_runlib"],  &["sh", "-c", "echo 'in_toto says hi' >> hello_intoto"], Some(&key), Some(&["sha512", "sha256"]), Some(&["tests/test_runlib/"])).unwrap();
 /// let json = serde_json::to_value(&link).unwrap();
 /// println!("Generated link: {}", json);
 /// ```
@@ -238,16 +289,17 @@ pub fn in_toto_run(
     cmd_args: &[&str],
     key: Option<&PrivateKey>,
     hash_algorithms: Option<&[&str]>,
+    lstrip_paths: Option<&[&str]>,
     // env: Option<BTreeMap<String, String>>
 ) -> Result<Metablock<Json, LinkMetadata>> {
     // Record Materials: Given the material_paths, recursively traverse and record files in given path(s)
-    let materials = record_artifacts(material_paths, hash_algorithms)?;
+    let materials = record_artifacts(material_paths, hash_algorithms, lstrip_paths)?;
 
     // Execute commands provided in cmd_args
     let byproducts = run_command(cmd_args, run_dir)?;
 
     // Record Products: Given the product_paths, recursively traverse and record files in given path(s)
-    let products = record_artifacts(product_paths, hash_algorithms)?;
+    let products = record_artifacts(product_paths, hash_algorithms, lstrip_paths)?;
 
     // Create link based on values collected above
     let link_metadata_builder = LinkMetadataBuilder::new()
@@ -368,16 +420,94 @@ mod test {
             ),
         );
         assert_eq!(
-            record_artifacts(&["tests/test_runlib"], None).unwrap(),
+            record_artifacts(&["tests/test_runlib"], None, None).unwrap(),
             expected
         );
-        assert_eq!(record_artifacts(&["tests"], None).is_ok(), true);
+        assert_eq!(record_artifacts(&["tests"], None, None).is_ok(), true);
         assert_eq!(
-            record_artifacts(&["file-does-not-exist"], None).is_err(),
+            record_artifacts(&["file-does-not-exist"], None, None).is_err(),
             true
         );
     }
 
+    #[test]
+    fn test_prefix_record_artifacts() {
+        let mut expected: BTreeMap<VirtualTargetPath, TargetDescription> = BTreeMap::new();
+        expected.insert(
+            VirtualTargetPath::new("world".to_string()).unwrap(),
+            create_target_description(
+                crypto::HashAlgorithm::Sha256,
+                b"25623b53e0984428da972f4c635706d32d01ec92dcd2ab39066082e0b9488c9d",
+            ),
+        );
+        assert_eq!(
+            record_artifacts(
+                &["tests/test_prefix/left"],
+                None,
+                Some(&["tests/test_prefix/left/"])
+            )
+            .unwrap(),
+            expected
+        );
+        // conflict of file "left/world" and "right/world"
+        assert_eq!(
+            record_artifacts(
+                &["tests/test_prefix"],
+                None,
+                Some(&["tests/test_prefix/left/", "tests/test_prefix/right/"])
+            )
+            .is_err(),
+            true
+        );
+    }
+
+    #[test]
+    fn test_left_strip() {
+        let mut stripped_path: String;
+
+        stripped_path = apply_left_strip(
+            "tests/test_runlib/.hidden/foo",
+            Some(&["tests/test_runlib"]),
+        )
+        .unwrap();
+        assert_eq!(stripped_path, "/.hidden/foo");
+
+        stripped_path = apply_left_strip(
+            "tests/test_runlib/.hidden/foo",
+            Some(&["tests/test_runlib/"]),
+        )
+        .unwrap();
+        assert_eq!(stripped_path, ".hidden/foo");
+
+        stripped_path = apply_left_strip(
+            "tests/test_runlib/.hidden/foo",
+            Some(&["tests/test_runlib/.hidden/"]),
+        )
+        .unwrap();
+        assert_eq!(stripped_path, "foo");
+
+        stripped_path = apply_left_strip(
+            "tests/test_runlib/.hidden/foo",
+            Some(&["path-does-not-exist"]),
+        )
+        .unwrap();
+        assert_eq!(stripped_path, "tests/test_runlib/.hidden/foo");
+
+        stripped_path = apply_left_strip(
+            "tests/test_runlib/.hidden/foo",
+            Some(&["path-does-not-exist", "tests/"]),
+        )
+        .unwrap();
+        assert_eq!(stripped_path, "test_runlib/.hidden/foo");
+
+        stripped_path = apply_left_strip(
+            "tests/test_runlib/.hidden/foo",
+            Some(&["tests/", "tests/test_runlib/.hidden/"]),
+        )
+        .unwrap();
+        assert_eq!(stripped_path, "foo");
+    }
+
     #[test]
     fn test_run_command() {
         let byproducts = run_command(&["sh", "-c", "printf hello"], Some("tests")).unwrap();
